// Copyright (c) 2008  Future Technology Devices International Ltd.
//
// Module Name
//
//     MainForm.cs
//
// Abstract:
//
//    C# program, to test the FTCI2C.DLL. The FTCI2C.DLL is used to control the FT2232H dual hi-speed device
//    and the FT4232H quad hi-speed device to simulate the Inter-Integrated Circuit(I2C) synchronous protocol to
//    communicate with I2C devices connected to FT2232H dual hi-speed devices and FT4232H quad hi-speed devices.
//
// Revision History:
//
//   06/08/08    Kirke Ashton    Created.
//   03/09/08    Kirke Ashton    Added new thread to test updated FTCI2C.DLL, to ensure it can handle multiple threads.
//
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.Threading;

namespace FT2232HI2CCSharpNETTestApp
{
    public partial class MainForm : Form
    {
        private const string App_Title = "FT2232/FT4232 I2C Device C# .NET Test Application";

        private const string Dll_Version_Label = "FT2232/FT4232 I2C DLL Version = ";

        private const string Device_Name_Label = "Device Name = ";

        private enum HI_SPEED_DEVICE_TYPES
        {
            FT2232H_DEVICE_TYPE = 1,
            FT4232H_DEVICE_TYPE = 2
        };

        private const uint FTC_SUCCESS = 0;
        private const uint FTC_DEVICE_IN_USE = 27;

        private const uint MAX_NUM_DEVICE_NAME_CHARS = 100;
        private const uint MAX_NUM_CHANNEL_CHARS = 5;

        private const uint MAX_NUM_DLL_VERSION_CHARS = 10;
        private const uint MAX_NUM_ERROR_MESSAGE_CHARS = 100;

        // To communicate with the M24C64(8192 byte) EEPROM, the maximum frequency the clock can be set is 375KHz 
        private const uint MAX_FREQ_M24C64_CLOCK_DIVISOR = 79;  // equivalent to 375KHz

        private const uint STANDARD_MODE = 1;
        private const uint FAST_MODE = 2;
        private const uint STRETCH_DATA_MODE = 4;

        private const uint WRITE_CONTROL_BUFFER_SIZE = 256;
        private const uint WRITE_DATA_BUFFER_SIZE = 65536;
        private const uint READ_DATA_BUFFER_SIZE = 65536;

        private const uint MAX_I2C_M24C64_CHIP_SIZE_IN_BYTES = 512;

        private const uint NUM_M24C64_PAGES = 1;
        private const uint NUM_M24C64_BYTES_PER_PAGE = 32;

        private const uint MAX_I2C_24LC16B_CHIP_SIZE_IN_BYTES = 256;

        private const uint NUM_24LC16B_PAGES = 8;
        private const uint NUM_24LC16B_BYTES_PER_PAGE = 256;

        private const uint NO_WRITE_TYPE = 0;
        private const uint BYTE_WRITE_TYPE = 1;
        private const uint PAGE_WRITE_TYPE = 2;

        private const uint BYTE_READ_TYPE = 1;
        private const uint PAGE_READ_TYPE = 2;


        //**************************************************************************
        //
        // TYPE DEFINITIONS
        //
        //**************************************************************************

        public struct FTC_INPUT_OUTPUT_PINS
        {
            public bool bPin1InputOutputState;
            public bool bPin1LowHighState;
            public bool bPin2InputOutputState;
            public bool bPin2LowHighState;
            public bool bPin3InputOutputState;
            public bool bPin3LowHighState;
            public bool bPin4InputOutputState;
            public bool bPin4LowHighState;
        }

        public struct FTH_INPUT_OUTPUT_PINS
        {
            public bool bPin1InputOutputState;
            public bool bPin1LowHighState;
            public bool bPin2InputOutputState;
            public bool bPin2LowHighState;
            public bool bPin3InputOutputState;
            public bool bPin3LowHighState;
            public bool bPin4InputOutputState;
            public bool bPin4LowHighState;
            public bool bPin5InputOutputState;
            public bool bPin5LowHighState;
            public bool bPin6InputOutputState;
            public bool bPin6LowHighState;
            public bool bPin7InputOutputState;
            public bool bPin7LowHighState;
            public bool bPin8InputOutputState;
            public bool bPin8LowHighState;
        }

        public struct FTC_LOW_HIGH_PINS
        {
            public bool bPin1LowHighState;
            public bool bPin2LowHighState;
            public bool bPin3LowHighState;
            public bool bPin4LowHighState;
        }

        public struct FTC_PAGE_WRITE_DATA
        {
            public uint NumPages;
            public uint NumBytesPerPage;
        }

        public struct FTH_LOW_HIGH_PINS
        {
            public bool bPin1LowHighState;
            public bool bPin2LowHighState;
            public bool bPin3LowHighState;
            public bool bPin4LowHighState;
            public bool bPin5LowHighState;
            public bool bPin6LowHighState;
            public bool bPin7LowHighState;
            public bool bPin8LowHighState;
        }

        public struct FTC_CLOSE_FINAL_STATE_PINS
        {
            public bool bTCKPinState;
            public bool bTCKPinActiveState;
            public bool bTDIPinState;
            public bool bTDIPinActiveState;
            public bool bTMSPinState;
            public bool bTMSPinActiveState;
        }


        //**************************************************************************
        //
        // FUNCTION IMPORTS FROM FTCI2C DLL
        //
        //**************************************************************************

        // Built-in Windows API functions to allow us to dynamically load our own DLL.
        [DllImportAttribute("ftci2c.dll", EntryPoint = "I2C_GetDllVersion", CallingConvention = CallingConvention.Cdecl)]
        static extern uint GetDllVersion(byte[] pDllVersion, uint buufferSize);

        [DllImportAttribute("ftci2c.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern uint I2C_GetErrorCodeString(string language, uint statusCode, byte[] pErrorMessage, uint bufferSize);

        [DllImportAttribute("ftci2c.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern uint I2C_GetNumHiSpeedDevices(ref uint NumHiSpeedDevices);

        [DllImportAttribute("ftci2c.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern uint I2C_GetHiSpeedDeviceNameLocIDChannel(uint deviceNameIndex, byte[] pDeviceName, uint deviceNameBufferSize, ref uint locationID, byte[] pChannel, uint channelBufferSize, ref uint hiSpeedDeviceType);

        [DllImportAttribute("ftci2c.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern uint I2C_OpenHiSpeedDevice(string DeviceName, uint locationID, string channel, ref IntPtr pftHandle);

        [DllImportAttribute("ftci2c.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern uint I2C_GetHiSpeedDeviceType(IntPtr ftHandle, ref uint hiSpeedDeviceType);

        [DllImportAttribute("ftci2c.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern uint I2C_Close(IntPtr ftHandle);

        [DllImportAttribute("ftci2c.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern uint I2C_CloseDevice(IntPtr ftHandle, ref FTC_CLOSE_FINAL_STATE_PINS pCloseFinalStatePinsData);

        [DllImportAttribute("ftci2c.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern uint I2C_InitDevice(IntPtr ftHandle, uint clockDivisor);

        [DllImportAttribute("ftci2c.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern uint I2C_TurnOnDivideByFiveClockingHiSpeedDevice(IntPtr ftHandle);

        [DllImportAttribute("ftci2c.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern uint I2C_TurnOffDivideByFiveClockingHiSpeedDevice(IntPtr ftHandle);

        [DllImportAttribute("ftci2c.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern uint I2C_TurnOnThreePhaseDataClockingHiSpeedDevice(IntPtr ftHandle);

        [DllImportAttribute("ftci2c.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern uint I2C_TurnOffThreePhaseDataClockingHiSpeedDevice(IntPtr ftHandle);

        [DllImportAttribute("ftci2c.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern uint I2C_SetDeviceLatencyTimer(IntPtr ftHandle, byte timerValue);

        [DllImportAttribute("ftci2c.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern uint I2C_GetDeviceLatencyTimer(IntPtr ftHandle, ref byte timerValue);

        [DllImportAttribute("ftci2c.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern uint I2C_GetHiSpeedDeviceClock(uint ClockDivisor, ref uint clockFrequencyHz);

        [DllImportAttribute("ftci2c.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern uint I2C_GetClock(uint clockDivisor, ref uint clockFrequencyHz);

        [DllImportAttribute("ftci2c.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern uint I2C_SetClock(IntPtr ftHandle, uint clockDivisor, ref uint clockFrequencyHz);

        [DllImportAttribute("ftci2c.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern uint I2C_SetLoopback(IntPtr ftHandle, bool bLoopBackState);

        [DllImportAttribute("ftci2c.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern uint I2C_SetMode(IntPtr ftHandle, uint commsMode);

        [DllImportAttribute("ftci2c.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern uint I2C_SetHiSpeedDeviceGPIOs(IntPtr ftHandle, bool bControlLowInputOutputPins, ref FTC_INPUT_OUTPUT_PINS pLowInputOutputPinsData, bool bControlHighInputOutputPins, ref FTH_INPUT_OUTPUT_PINS pHighInputOutputPinsData);

        [DllImportAttribute("ftci2c.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern uint I2C_GetHiSpeedDeviceGPIOs(IntPtr ftHandle, bool bControlLowInputOutputPins, out FTC_LOW_HIGH_PINS pLowPinsInputData, bool bControlHighInputOutputPins, out FTH_LOW_HIGH_PINS pHighPinsInputData);

        [DllImportAttribute("ftci2c.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern uint I2C_Write(IntPtr ftHandle, byte[] WriteControlBuffer, uint numControlBytesToWrite, bool bControlAcknowledge, uint controlAckTimeoutmSecs, bool bStopCondition, uint dataWriteType, byte[] WriteDataBuffer, uint numDataBytesToWrite, bool bDataAcknowledge, uint dataAckTimeoutmSecs, ref FTC_PAGE_WRITE_DATA PageWriteData);

        [DllImportAttribute("ftci2c.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern uint I2C_Read(IntPtr ftHandle, byte[] WriteControlBuffer, uint numControlBytesToWrite, bool bControlAcknowledge, uint controlAckTimeoutmSecs, uint dataReadType, byte[] ReadDataBuffer, uint numDataBytesToRead);

        public MainForm()
        {
            InitializeComponent();

            this.Text = App_Title;

            this.ChannelComboBox.Items.Clear();
            this.ChannelComboBox.Items.Add("A");
            this.ChannelComboBox.Items.Add("B");
            this.ChannelComboBox.Text = "A";
            //this.ChannelComboBox.SelectedIndex = -1;
        }

        private uint TestHiSpeedDeviceClock(IntPtr ftHandle, string threadName)
        {
            uint ftStatus = FTC_SUCCESS;
            uint clockFrequencyHz = 0;

            if ((ftStatus = I2C_GetHiSpeedDeviceClock(0, ref clockFrequencyHz)) == FTC_SUCCESS)
            {
                if ((ftStatus = I2C_TurnOnDivideByFiveClockingHiSpeedDevice(ftHandle)) == FTC_SUCCESS)
                {
                    ftStatus = I2C_GetHiSpeedDeviceClock(0, ref clockFrequencyHz);

                    if ((ftStatus = I2C_SetClock(ftHandle, MAX_FREQ_M24C64_CLOCK_DIVISOR, ref clockFrequencyHz)) == FTC_SUCCESS)
                    {
                        ftStatus = I2C_TurnOffDivideByFiveClockingHiSpeedDevice(ftHandle);

                        ftStatus = I2C_SetClock(ftHandle, MAX_FREQ_M24C64_CLOCK_DIVISOR, ref clockFrequencyHz);
                    }
                }
            }

            return ftStatus;
        }

        public void ThreadProc()
        {
            uint ftStatus = FTC_SUCCESS;
            IntPtr ftHandleThread = IntPtr.Zero;
            uint numHiSpeedDevices = 0; // 32-bit unsigned integer
            uint hiSpeedDeviceIndex = 0;
            uint locationID = 0;
            byte[] byteHiSpeedDeviceName = new byte[MAX_NUM_DEVICE_NAME_CHARS];
            byte[] byteHiSpeedDeviceChannel = new byte[MAX_NUM_CHANNEL_CHARS];
            uint hiSpeedDeviceType = 0;
            string hiSpeedDeviceName = null;
            string hiSpeedChannel = null;
            byte timerValue = 0;
            uint clockFrequencyHz = 0;
            bool bLoopBackState = false;
            FTC_INPUT_OUTPUT_PINS LowInputOutputPinsData = new FTC_INPUT_OUTPUT_PINS();
            FTH_INPUT_OUTPUT_PINS HighInputOutputPinsData = new FTH_INPUT_OUTPUT_PINS();
            FTC_LOW_HIGH_PINS LowPinsInputData = new FTC_LOW_HIGH_PINS(); FTH_LOW_HIGH_PINS HighPinsInputData = new FTH_LOW_HIGH_PINS();
            uint WriteDataByteAddress = 0;
            uint DataWriteType = BYTE_WRITE_TYPE;
            byte DataByte = 0;
            uint NumDataBytesToWrite = 0;
            int WriteDataBufferIndex = 0;
            byte[] WriteControlBuffer = new byte[WRITE_CONTROL_BUFFER_SIZE];
            byte[] WriteDataBuffer = new byte[WRITE_DATA_BUFFER_SIZE];
            FTC_PAGE_WRITE_DATA PageWriteData;
            int ReadDataIndex = 0;
            uint ReadDataByteAddress = 0;
            byte[] ReadDataBuffer = new byte[READ_DATA_BUFFER_SIZE];
            byte[] ReadData = new byte[READ_DATA_BUFFER_SIZE];
            uint DataReadType = 0;
            uint NumDataBytesToRead = 0;
            byte DataByteWritten = 0;
            int CharCntr = 0;
            int iLoopCntr = 0;
            string mismatchMsg = null;
            MessageBoxButtons MsgBoxButtons = MessageBoxButtons.OKCancel;
            DialogResult response = DialogResult.OK;

            byte[] byteErrorMessage = new byte[MAX_NUM_ERROR_MESSAGE_CHARS];
            String ErrorMessage = null;

            //Console.WriteLine("Second thread started.");

            // The parameter of this function is a pointer to a variable of type DWORD ie 32-bit unsigned integer
            ftStatus = I2C_GetNumHiSpeedDevices(ref numHiSpeedDevices);

            if ((ftStatus == FTC_SUCCESS) && (numHiSpeedDevices > 0))
            {
                //Console.WriteLine("Second thread - get number of devices.");

                do
                {
                    ftStatus = I2C_GetHiSpeedDeviceNameLocIDChannel(hiSpeedDeviceIndex, byteHiSpeedDeviceName, MAX_NUM_DEVICE_NAME_CHARS, ref locationID, byteHiSpeedDeviceChannel, MAX_NUM_CHANNEL_CHARS, ref hiSpeedDeviceType);

                    
                    if (ftStatus == FTC_SUCCESS)
                    {
                        //Console.WriteLine("Second thread - get device name and channel.");

                        hiSpeedChannel = Encoding.ASCII.GetString(byteHiSpeedDeviceChannel);
                        // Trim strings to first occurrence of a null terminator character
                        hiSpeedChannel = hiSpeedChannel.Substring(0, hiSpeedChannel.IndexOf("\0"));
                    }

                    hiSpeedDeviceIndex = hiSpeedDeviceIndex + 1;
                }
                while ((ftStatus == FTC_SUCCESS) && (hiSpeedDeviceIndex < numHiSpeedDevices) && ((hiSpeedChannel != null) && (hiSpeedChannel != "B")));

                if (ftStatus == FTC_SUCCESS)
                {
                    if ((hiSpeedChannel != null) && (hiSpeedChannel != "B"))
                        ftStatus = FTC_DEVICE_IN_USE;
                }

                if (ftStatus == FTC_SUCCESS)
                {
                    hiSpeedDeviceName = Encoding.ASCII.GetString(byteHiSpeedDeviceName);
                    // Trim strings to first occurrence of a null terminator character
                    hiSpeedDeviceName = hiSpeedDeviceName.Substring(0, hiSpeedDeviceName.IndexOf("\0"));

                    // The ftHandle parameter is a pointer to a variable of type DWORD ie 32-bit unsigned integer
                    ftStatus = I2C_OpenHiSpeedDevice(hiSpeedDeviceName, locationID, hiSpeedChannel, ref ftHandleThread);

                    if (ftStatus == FTC_SUCCESS)
                    {
                        //Console.WriteLine("Second thread - opened device.");

                        ftStatus = I2C_GetHiSpeedDeviceType(ftHandleThread, ref hiSpeedDeviceType);

                        //Console.WriteLine("Second thread - get device type.");
                    }
                }

                if ((ftHandleThread != IntPtr.Zero) && (ftStatus == FTC_SUCCESS))
                {
                    ftStatus = I2C_InitDevice(ftHandleThread, MAX_FREQ_M24C64_CLOCK_DIVISOR);

                    if (ftStatus == FTC_SUCCESS)
                    {
                        //Console.WriteLine("Second thread - init device.");

                        if ((ftStatus = I2C_GetDeviceLatencyTimer(ftHandleThread, ref timerValue)) == FTC_SUCCESS)
                        {
                            if ((ftStatus = I2C_SetDeviceLatencyTimer(ftHandleThread, 50)) == FTC_SUCCESS)
                            {
                                ftStatus = I2C_GetDeviceLatencyTimer(ftHandleThread, ref timerValue);

                                ftStatus = I2C_SetDeviceLatencyTimer(ftHandleThread, 16);

                                ftStatus = I2C_GetDeviceLatencyTimer(ftHandleThread, ref timerValue);
                            }
                        }
                    }

                    if (ftStatus == FTC_SUCCESS)
                    {
                        //Console.WriteLine("Second thread - device latency timer.");

                        ftStatus = TestHiSpeedDeviceClock(ftHandleThread, "Second thread");
                    }

                    if (ftStatus == FTC_SUCCESS)
                    {
                        //Console.WriteLine("Second thread - device clock.");

                        ftStatus = I2C_SetLoopback(ftHandleThread, bLoopBackState);
                    }

                    if (ftStatus == FTC_SUCCESS)
                    {
                        ftStatus = I2C_SetMode(ftHandleThread, FAST_MODE);

                        //Console.WriteLine("Second thread - set mode.");

                        //Thread.Sleep(0);
                    }

                    if (ftStatus == FTC_SUCCESS)
                    {
                        LowInputOutputPinsData.bPin1InputOutputState = false;
                        LowInputOutputPinsData.bPin1LowHighState = false;

                        LowInputOutputPinsData.bPin2InputOutputState = true;
                        LowInputOutputPinsData.bPin2LowHighState = true;
                        LowInputOutputPinsData.bPin3InputOutputState = true;
                        LowInputOutputPinsData.bPin3LowHighState = false;
                        LowInputOutputPinsData.bPin4InputOutputState = true;
                        LowInputOutputPinsData.bPin4LowHighState = true;

                        // Set the 8 general purpose higher input/output pins has to input or output
                        HighInputOutputPinsData.bPin1InputOutputState = true;
                        HighInputOutputPinsData.bPin1LowHighState = true;
                        HighInputOutputPinsData.bPin2InputOutputState = true;
                        HighInputOutputPinsData.bPin2LowHighState = false;
                        HighInputOutputPinsData.bPin3InputOutputState = true;
                        HighInputOutputPinsData.bPin3LowHighState = true;
                        HighInputOutputPinsData.bPin4InputOutputState = true;
                        HighInputOutputPinsData.bPin4LowHighState = false;
                        HighInputOutputPinsData.bPin5InputOutputState = true;
                        HighInputOutputPinsData.bPin5LowHighState = true;
                        HighInputOutputPinsData.bPin6InputOutputState = true;
                        HighInputOutputPinsData.bPin6LowHighState = false;
                        HighInputOutputPinsData.bPin7InputOutputState = true;
                        HighInputOutputPinsData.bPin7LowHighState = true;
                        HighInputOutputPinsData.bPin8InputOutputState = true;
                        HighInputOutputPinsData.bPin8LowHighState = false;

                        if (hiSpeedDeviceType == (uint)HI_SPEED_DEVICE_TYPES.FT2232H_DEVICE_TYPE)
                            ftStatus = I2C_SetHiSpeedDeviceGPIOs(ftHandleThread, true, ref LowInputOutputPinsData, true, ref HighInputOutputPinsData);
                        else
                            ftStatus = I2C_SetHiSpeedDeviceGPIOs(ftHandleThread, true, ref LowInputOutputPinsData, false, ref HighInputOutputPinsData);

                        if (ftStatus == FTC_SUCCESS)
                        {
                            //Console.WriteLine("Second thread - set device GPIOs.");

                            // Sleep for 2 milli-seconds
                            Thread.Sleep(2);

                            if (hiSpeedDeviceType == (uint)HI_SPEED_DEVICE_TYPES.FT2232H_DEVICE_TYPE)
                                ftStatus = I2C_GetHiSpeedDeviceGPIOs(ftHandleThread, true, out LowPinsInputData, true, out HighPinsInputData);
                            else
                                ftStatus = I2C_GetHiSpeedDeviceGPIOs(ftHandleThread, true, out LowPinsInputData, false, out HighPinsInputData);


                            if (ftStatus == FTC_SUCCESS)
                            {
                                //Console.WriteLine("Second thread - get device GPIOs.");
                            }
                        }
                    }

                    if ((ftStatus == FTC_SUCCESS) && (this.GPIOL1To3GPIOH0To7OnlyCheckBox.Checked == false))
                    {
                        WriteControlBuffer[0] = 0xA2; //1 0 1 0 A2 A1 A0 0 - device address 1

                        WriteDataBuffer[0] = 0xFF;  // Erase address locations

                        WriteDataByteAddress = 0;

                        PageWriteData.NumPages = 0;
                        PageWriteData.NumBytesPerPage = 0;

                        do
                        {
                            // shift down by 8 bits
                            WriteControlBuffer[1] = (byte)((WriteDataByteAddress >> 8) & 0xFF);
                            WriteControlBuffer[2] = (byte)(WriteDataByteAddress & 0xFF);

                            ftStatus = I2C_Write(ftHandleThread, WriteControlBuffer, 3, true, 20, true, BYTE_WRITE_TYPE, WriteDataBuffer, 1, true, 20, ref PageWriteData);

                            WriteDataByteAddress = WriteDataByteAddress + 1;
                        }
                        while ((WriteDataByteAddress < MAX_I2C_24LC16B_CHIP_SIZE_IN_BYTES) && (ftStatus == FTC_SUCCESS));
                    }

                    //Console.WriteLine("Second thread - erased EEPROM.");

                    if ((ftStatus == FTC_SUCCESS) && (this.GPIOL1To3GPIOH0To7OnlyCheckBox.Checked == false))
                    {
                        //WriteControlBuffer[0] = 0xAE; //1 0 1 0 A2 A1 A0 0 - device address 7
                        WriteControlBuffer[0] = 0xA2; //1 0 1 0 A2 A1 A0 0 - device address 1

                        WriteDataByteAddress = 0;

                        DataWriteType = BYTE_WRITE_TYPE; //PAGE_WRITE_TYPE;

                        DataByte = 0x77;

                        if (DataWriteType == BYTE_WRITE_TYPE)
                        {
                            WriteDataBuffer[0] = DataByte;

                            PageWriteData.NumPages = 0;
                            PageWriteData.NumBytesPerPage = 0;

                            NumDataBytesToWrite = 1;
                        }
                        else
                        {
                            WriteDataBuffer[0] = DataByte;

                            PageWriteData.NumPages = NUM_24LC16B_PAGES;
                            PageWriteData.NumBytesPerPage = NUM_24LC16B_BYTES_PER_PAGE;

                            NumDataBytesToWrite = (NUM_24LC16B_PAGES * NUM_24LC16B_BYTES_PER_PAGE);

                            for (WriteDataBufferIndex = 0; WriteDataBufferIndex < NumDataBytesToWrite; WriteDataBufferIndex++)
                            {
                                WriteDataBuffer[WriteDataBufferIndex] = DataByte;
                            }
                        }

                        do
                        {
                            // performs an arithmetic right shift down by 8 bits
                            WriteControlBuffer[1] = (byte)((WriteDataByteAddress >> 8) & 0xFF);
                            WriteControlBuffer[2] = (byte)(WriteDataByteAddress & 0xFF);

                            ftStatus = I2C_Write(ftHandleThread, WriteControlBuffer, 3, true, 20, true, DataWriteType, WriteDataBuffer, NumDataBytesToWrite, true, 20, ref PageWriteData);

                            if (DataWriteType == BYTE_WRITE_TYPE)
                            {
                                WriteDataByteAddress = WriteDataByteAddress + 1;
                            }
                            else
                            {
                                WriteDataByteAddress = WriteDataByteAddress + (NUM_24LC16B_PAGES * NUM_24LC16B_BYTES_PER_PAGE);
                            }
                        }
                        while ((WriteDataByteAddress < MAX_I2C_24LC16B_CHIP_SIZE_IN_BYTES) && (ftStatus == FTC_SUCCESS));

                        //Console.WriteLine("Second thread - values written to EEPROM.");

                        if (ftStatus == FTC_SUCCESS)
                        {
                            do
                            {
                                //WriteControlBuffer[0] = 0xAF; //1 0 1 0 A2 A1 A0 1 - device address 7
                                WriteControlBuffer[0] = 0xA3; //1 0 1 0 A2 A1 A0 1 - device address 1

                                for (ReadDataIndex = 0; ReadDataIndex < MAX_I2C_24LC16B_CHIP_SIZE_IN_BYTES; ReadDataIndex++)
                                {
                                    ReadData[ReadDataIndex] = 0;
                                }

                                ReadDataByteAddress = 0;

                                DataReadType = BYTE_READ_TYPE; //BLOCK_READ_TYPE

                                if (DataReadType == BYTE_READ_TYPE)
                                {
                                    NumDataBytesToRead = 1;
                                }
                                else
                                {
                                    NumDataBytesToRead = MAX_I2C_24LC16B_CHIP_SIZE_IN_BYTES;
                                }

                                do
                                {
                                    // performs an arithmetic right shift down by 8 bits
                                    WriteControlBuffer[1] = (byte)((ReadDataByteAddress >> 8) & 0xFF);
                                    WriteControlBuffer[2] = (byte)(ReadDataByteAddress & 0xFF);

                                    ftStatus = I2C_Read(ftHandleThread, WriteControlBuffer, 3, true, 20, DataReadType, ReadDataBuffer, NumDataBytesToRead);

                                    if (ftStatus == FTC_SUCCESS)
                                    {
                                        if (DataReadType == BYTE_READ_TYPE)
                                        {
                                            ReadData[ReadDataByteAddress] = ReadDataBuffer[0];
                                        }
                                        else
                                        {
                                            for (ReadDataIndex = 0; ReadDataIndex < MAX_I2C_24LC16B_CHIP_SIZE_IN_BYTES; ReadDataIndex++)
                                            {
                                                ReadData[ReadDataIndex] = ReadDataBuffer[ReadDataIndex];
                                            }
                                        }

                                        if (DataReadType == BYTE_READ_TYPE)
                                        {
                                            ReadDataByteAddress = ReadDataByteAddress + 1;
                                        }
                                        else
                                        {
                                            ReadDataByteAddress = ReadDataByteAddress + MAX_I2C_24LC16B_CHIP_SIZE_IN_BYTES;
                                        }
                                    }
                                }
                                while ((ReadDataByteAddress < MAX_I2C_24LC16B_CHIP_SIZE_IN_BYTES) && (ftStatus == FTC_SUCCESS));

                                //Console.WriteLine("Second thread - values read from EEPROM.");

                                if (ftStatus == FTC_SUCCESS)
                                {
                                    ReadDataIndex = 0;

                                    do
                                    {
                                        if (DataByte != 0)
                                        {
                                            DataByteWritten = DataByte;
                                        }
                                        else
                                        {
                                            DataByteWritten = (byte)(ReadDataIndex & 0xFF);
                                        }

                                        if (((ReadData[ReadDataIndex] != DataByteWritten) && (DataByte != 0)) || ((ReadData[ReadDataIndex] != DataByteWritten) && (DataByte == 0)))
                                        {
                                            for (CharCntr = 0; CharCntr < 100; CharCntr++)
                                            {
                                                WriteDataBuffer[CharCntr] = 0;
                                            }

                                            mismatchMsg = String.Format("24LC16B Read/Write mismatch, Address = {0}, Write Value = {1}, Read Value = {2}", ReadDataIndex, DataByteWritten, ReadData[ReadDataIndex]);

                                            response = MessageBox.Show(mismatchMsg, "I2C Read Error Message", MsgBoxButtons);
                                        }

                                        ReadDataIndex = ReadDataIndex + 1;
                                    }
                                    while ((ReadDataIndex < MAX_I2C_24LC16B_CHIP_SIZE_IN_BYTES) && (response == DialogResult.OK));
                                }

                                iLoopCntr = iLoopCntr + 1;
                            }
                            while ((iLoopCntr < 10) && (ftStatus == FTC_SUCCESS) && (response == DialogResult.OK));
                        }
                    }
                }
            }

            if (ftHandleThread != IntPtr.Zero)
            {
                I2C_Close(ftHandleThread);
                ftHandleThread = IntPtr.Zero;

                //Console.WriteLine("Second thread - device closed.");
            }

            if ((ftStatus != FTC_SUCCESS) || (numHiSpeedDevices == 0) || (mismatchMsg != null))
            {
                if (ftStatus != FTC_SUCCESS) {
                    ErrorMessage = "Status Code(" + ftStatus.ToString() + ") - ";

                    ftStatus = I2C_GetErrorCodeString("EN", ftStatus, byteErrorMessage, MAX_NUM_ERROR_MESSAGE_CHARS);

                    ErrorMessage = ErrorMessage + Encoding.ASCII.GetString(byteErrorMessage);
                    // Trim strings to first occurrence of a null terminator character
                    ErrorMessage = ErrorMessage.Substring(0, ErrorMessage.IndexOf("\0"));
                } else {
                    if (numHiSpeedDevices == 0)
                    {
                        ErrorMessage = "There are no devices connected.";
                    }
                    else
                    {
                        ErrorMessage = mismatchMsg;
                    }
                }

                MessageBox.Show(ErrorMessage);

                //Console.WriteLine(ErrorMessage);
                Console.WriteLine("Second Thread Fail - {0}", ErrorMessage);
            } else {
                MessageBox.Show("Second Thread - Pass");
            }
        }

        private void OKbutton_Click(object sender, EventArgs e)
        {
            uint ftStatus = FTC_SUCCESS;
            uint numHiSpeedDevices = 0; // 32-bit unsigned integer
            uint hiSpeedDeviceIndex = 0;
            uint locationID = 0;
            byte[] byteHiSpeedDeviceName = new byte[MAX_NUM_DEVICE_NAME_CHARS];
            byte[] byteHiSpeedDeviceChannel = new byte[MAX_NUM_CHANNEL_CHARS];
            uint hiSpeedDeviceType = 0;
            string hiSpeedDeviceName = null;
            string hiSpeedChannel = null;
            IntPtr ftHandle = IntPtr.Zero;
            byte timerValue = 0;
            uint clockFrequencyHz = 0;
            bool bLoopBackState = false;
            FTC_INPUT_OUTPUT_PINS LowInputOutputPinsData = new FTC_INPUT_OUTPUT_PINS();
            FTH_INPUT_OUTPUT_PINS HighInputOutputPinsData = new FTH_INPUT_OUTPUT_PINS();
            FTC_LOW_HIGH_PINS LowPinsInputData = new FTC_LOW_HIGH_PINS();
            FTH_LOW_HIGH_PINS HighPinsInputData = new FTH_LOW_HIGH_PINS();
            FTC_CLOSE_FINAL_STATE_PINS CloseFinalStatePinsData;
            uint WriteDataByteAddress = 0;
            uint DataWriteType = BYTE_WRITE_TYPE;
            byte DataByte = 0;
            uint NumDataBytesToWrite = 0;
            int WriteDataBufferIndex = 0;
            byte[] WriteControlBuffer = new byte[WRITE_CONTROL_BUFFER_SIZE];
            byte[] WriteDataBuffer = new byte[WRITE_DATA_BUFFER_SIZE];
            FTC_PAGE_WRITE_DATA PageWriteData;
            int ReadDataIndex = 0;
            uint ReadDataByteAddress = 0;
            byte[] ReadDataBuffer = new byte[READ_DATA_BUFFER_SIZE];
            byte[] ReadData = new byte[READ_DATA_BUFFER_SIZE];
            uint DataReadType = 0;
            uint NumDataBytesToRead = 0;
            byte DataByteWritten = 0;
            int CharCntr = 0;
            int iLoopCntr = 0;
            string mismatchMsg = null;
            MessageBoxButtons MsgBoxButtons = MessageBoxButtons.OKCancel;
            DialogResult response = DialogResult.OK;

            byte[] byteDllVersion = new byte[MAX_NUM_DLL_VERSION_CHARS];
            string DllVersion;
            byte[] byteErrorMessage = new byte[MAX_NUM_ERROR_MESSAGE_CHARS];
            String ErrorMessage = null;

            this.GPIOL1InputLowHighReadStateCheckBox.Checked = false;
            this.GPIOL2InputLowHighReadStateCheckBox.Checked = false;
            this.GPIOL3InputLowHighReadStateCheckBox.Checked = false;

            this.GPIOH0InputLowHighReadStateCheckBox.Checked = false;
            this.GPIOH1InputLowHighReadStateCheckBox.Checked = false;
            this.GPIOH2InputLowHighReadStateCheckBox.Checked = false;
            this.GPIOH3InputLowHighReadStateCheckBox.Checked = false;
            this.GPIOH4InputLowHighReadStateCheckBox.Checked = false;
            this.GPIOH5InputLowHighReadStateCheckBox.Checked = false;
            this.GPIOH6InputLowHighReadStateCheckBox.Checked = false;
            this.GPIOH7InputLowHighReadStateCheckBox.Checked = false;
 
            // This function returns a ULONG value ie 32-bit unsigned integer
            //ftStatus = JTAG_GetDllVersion(byteDllVersion, MAX_NUM_DLL_VERSION_CHARS);
            ftStatus = GetDllVersion(byteDllVersion, MAX_NUM_DLL_VERSION_CHARS);

            DllVersion = Encoding.ASCII.GetString(byteDllVersion);
            // Trim strings to first occurrence of a null terminator character
            DllVersion = DllVersion.Substring(0, DllVersion.IndexOf("\0"));

            this.DLLVersionLabel.Text = Dll_Version_Label + DllVersion;
            this.DLLVersionLabel.Refresh();

            this.DeviceNameLabel.Text = Device_Name_Label;
            this.DeviceNameLabel.Refresh();

            this.FT2232HRadioButton.Checked = false;
            this.FT4232HRadioButton.Checked = false;

            this.PassFailResultsStatusLabel.Text = "";
            this.PassFailureStatusStrip.Refresh();
            Thread.Sleep(100);

            if (this.MultipleThreadingCheckBox.Checked == true)
            {
                // The constructor for the Thread class requires a ThreadStart delegate
                // that represents the method to be executed on the thread. 
                Thread t = new Thread(new ThreadStart(ThreadProc));

                // Start ThreadProc.  Note that on a uniprocessor, the new thread does not
                // get any processor time until the main thread is preempted or yields.
                // Uncomment the Thread.Sleep that follows t.Start() to see the difference.
                t.Start();
                //Thread.Sleep(0);
            }

            // The parameter of this function is a pointer to a variable of type DWORD ie 32-bit unsigned integer
            ftStatus = I2C_GetNumHiSpeedDevices(ref numHiSpeedDevices);

            if ((ftStatus == FTC_SUCCESS) && (numHiSpeedDevices > 0))
            {
                //Console.WriteLine("Main thread get number of devices.");

                do
                {
                    ftStatus = I2C_GetHiSpeedDeviceNameLocIDChannel(hiSpeedDeviceIndex, byteHiSpeedDeviceName, MAX_NUM_DEVICE_NAME_CHARS, ref locationID, byteHiSpeedDeviceChannel, MAX_NUM_CHANNEL_CHARS, ref hiSpeedDeviceType);

                    if (ftStatus == FTC_SUCCESS)
                    {
                        hiSpeedChannel = Encoding.ASCII.GetString(byteHiSpeedDeviceChannel);
                        // Trim strings to first occurrence of a null terminator character
                        hiSpeedChannel = hiSpeedChannel.Substring(0, hiSpeedChannel.IndexOf("\0"));
                    }

                    hiSpeedDeviceIndex = hiSpeedDeviceIndex + 1;
                }
                while ((ftStatus == FTC_SUCCESS) && (hiSpeedDeviceIndex < numHiSpeedDevices) && ((hiSpeedChannel != null) && (hiSpeedChannel != this.ChannelComboBox.Text)));

                if (ftStatus == FTC_SUCCESS)
                {
                    if ((hiSpeedChannel != null) && (hiSpeedChannel != this.ChannelComboBox.Text))
                        ftStatus = FTC_DEVICE_IN_USE;
                }

                if (ftStatus == FTC_SUCCESS)
                {
                    hiSpeedDeviceName = Encoding.ASCII.GetString(byteHiSpeedDeviceName);
                    // Trim strings to first occurrence of a null terminator character
                    hiSpeedDeviceName = hiSpeedDeviceName.Substring(0, hiSpeedDeviceName.IndexOf("\0"));

                    // The ftHandle parameter is a pointer to a variable of type DWORD ie 32-bit unsigned integer
                    ftStatus = I2C_OpenHiSpeedDevice(hiSpeedDeviceName, locationID, hiSpeedChannel, ref ftHandle);

                    if (ftStatus == FTC_SUCCESS)
                    {
                        this.DeviceNameLabel.Text = Device_Name_Label + hiSpeedDeviceName;
                        this.DeviceNameLabel.Refresh();

                        ftStatus = I2C_GetHiSpeedDeviceType(ftHandle, ref hiSpeedDeviceType);

                        if (ftStatus == FTC_SUCCESS)
                        {
                            if (hiSpeedDeviceType == (uint)HI_SPEED_DEVICE_TYPES.FT4232H_DEVICE_TYPE)
                                this.FT4232HRadioButton.Checked = true;
                            else if (hiSpeedDeviceType == (uint)HI_SPEED_DEVICE_TYPES.FT2232H_DEVICE_TYPE)
                                this.FT2232HRadioButton.Checked = true;
                        }
                    }
                }

                if ((ftHandle != IntPtr.Zero) && (ftStatus == FTC_SUCCESS))
                {
                    ftStatus = I2C_InitDevice(ftHandle, MAX_FREQ_M24C64_CLOCK_DIVISOR);

                    if (ftStatus == FTC_SUCCESS)
                    {
                        //Console.WriteLine("Main thread - init device.");

                        //ftStatus = I2C_TurnOnThreePhaseDataClockingHiSpeedDevice(ftHandle);

                        //ftStatus = I2C_TurnOffThreePhaseDataClockingHiSpeedDevice(ftHandle);
                    }

                    if (ftStatus == FTC_SUCCESS)
                    {
                        if ((ftStatus = I2C_GetDeviceLatencyTimer(ftHandle, ref timerValue)) == FTC_SUCCESS)
                        {
                            if ((ftStatus = I2C_SetDeviceLatencyTimer(ftHandle, 50)) == FTC_SUCCESS)
                            {
                                ftStatus = I2C_GetDeviceLatencyTimer(ftHandle, ref timerValue);

                                ftStatus = I2C_SetDeviceLatencyTimer(ftHandle, 16);

                                ftStatus = I2C_GetDeviceLatencyTimer(ftHandle, ref timerValue);
                            }
                        }
                    }

                    if (ftStatus == FTC_SUCCESS)
                    {
                        //Console.WriteLine("Main thread - device latency timer.");

                        ftStatus = TestHiSpeedDeviceClock(ftHandle, "Main thread");

                        //Console.WriteLine("Main thread - device clock.");
                    }

                    if (ftStatus == FTC_SUCCESS)
                    {
                        ftStatus = I2C_SetLoopback(ftHandle, bLoopBackState);
                    }

                    if (ftStatus == FTC_SUCCESS) {
                        ftStatus = I2C_SetMode(ftHandle, FAST_MODE);
                    }

                    if (ftStatus == FTC_SUCCESS) {
                        //Console.WriteLine("Main thread - set mode.");

                        LowInputOutputPinsData.bPin1InputOutputState = false;
                        LowInputOutputPinsData.bPin1LowHighState = false;
                        LowInputOutputPinsData.bPin2InputOutputState = this.GPIOL1InputOutputInitialStateCheckBox.Checked;
                        LowInputOutputPinsData.bPin2LowHighState = this.GPIOL1OutputLowHighInitialStateCheckBox.Checked;
                        LowInputOutputPinsData.bPin3InputOutputState = this.GPIOL2InputOutputInitialStateCheckBox.Checked;
                        LowInputOutputPinsData.bPin3LowHighState = this.GPIOL2OutputLowHighInitialStateCheckBox.Checked;
                        LowInputOutputPinsData.bPin4InputOutputState = this.GPIOL3InputOutputInitialStateCheckBox.Checked;
                        LowInputOutputPinsData.bPin4LowHighState = this.GPIOL3OutputLowHighInitialStateCheckBox.Checked;

                        // Set the 8 general purpose higher input/output pins has to input or output
                        HighInputOutputPinsData.bPin1InputOutputState = this.GPIOH0InputOutputInitialStateCheckBox.Checked;
                        HighInputOutputPinsData.bPin1LowHighState = this.GPIOH0OutputLowHighInitialStateCheckBox.Checked;
                        HighInputOutputPinsData.bPin2InputOutputState = this.GPIOH1InputOutputInitialStateCheckBox.Checked;
                        HighInputOutputPinsData.bPin2LowHighState = this.GPIOH1OutputLowHighInitialStateCheckBox.Checked;
                        HighInputOutputPinsData.bPin3InputOutputState = this.GPIOH2InputOutputInitialStateCheckBox.Checked;
                        HighInputOutputPinsData.bPin3LowHighState = this.GPIOH2OutputLowHighInitialStateCheckBox.Checked;
                        HighInputOutputPinsData.bPin4InputOutputState = this.GPIOH3InputOutputInitialStateCheckBox.Checked;
                        HighInputOutputPinsData.bPin4LowHighState = this.GPIOH3OutputLowHighInitialStateCheckBox.Checked;
                        HighInputOutputPinsData.bPin5InputOutputState = this.GPIOH4InputOutputInitialStateCheckBox.Checked;
                        HighInputOutputPinsData.bPin5LowHighState = this.GPIOH4OutputLowHighInitialStateCheckBox.Checked;
                        HighInputOutputPinsData.bPin6InputOutputState = this.GPIOH5InputOutputInitialStateCheckBox.Checked;
                        HighInputOutputPinsData.bPin6LowHighState = this.GPIOH5OutputLowHighInitialStateCheckBox.Checked;
                        HighInputOutputPinsData.bPin7InputOutputState = this.GPIOH6InputOutputInitialStateCheckBox.Checked;
                        HighInputOutputPinsData.bPin7LowHighState = this.GPIOH6OutputLowHighInitialStateCheckBox.Checked;
                        HighInputOutputPinsData.bPin8InputOutputState = this.GPIOH7InputOutputInitialStateCheckBox.Checked;
                        HighInputOutputPinsData.bPin8LowHighState = this.GPIOH7OutputLowHighInitialStateCheckBox.Checked;

                        if (hiSpeedDeviceType == (uint)HI_SPEED_DEVICE_TYPES.FT2232H_DEVICE_TYPE)
                            ftStatus = I2C_SetHiSpeedDeviceGPIOs(ftHandle, true, ref LowInputOutputPinsData, true, ref HighInputOutputPinsData);
                        else
                            ftStatus = I2C_SetHiSpeedDeviceGPIOs(ftHandle, true, ref LowInputOutputPinsData, false, ref HighInputOutputPinsData);

                        if (ftStatus == FTC_SUCCESS) {
                            //Console.WriteLine("Main thread - set device GPIOs.");

                            // Sleep for 2 milli-seconds
                            Thread.Sleep(2);

                            if (hiSpeedDeviceType == (uint)HI_SPEED_DEVICE_TYPES.FT2232H_DEVICE_TYPE)
                                ftStatus = I2C_GetHiSpeedDeviceGPIOs(ftHandle, true, out LowPinsInputData, true, out HighPinsInputData);
                            else
                                ftStatus = I2C_GetHiSpeedDeviceGPIOs(ftHandle, true, out LowPinsInputData, false, out HighPinsInputData);

                            //Console.WriteLine("Main thread - get device GPIOs.");

                            if (ftStatus == FTC_SUCCESS)
                            {
                                this.GPIOL1InputLowHighReadStateCheckBox.Checked = LowPinsInputData.bPin2LowHighState;
                                this.GPIOL2InputLowHighReadStateCheckBox.Checked = LowPinsInputData.bPin3LowHighState;
                                this.GPIOL3InputLowHighReadStateCheckBox.Checked = LowPinsInputData.bPin4LowHighState;

                                if (hiSpeedDeviceType == (uint)HI_SPEED_DEVICE_TYPES.FT2232H_DEVICE_TYPE)
                                {
                                    this.GPIOH0InputLowHighReadStateCheckBox.Checked = HighPinsInputData.bPin1LowHighState;
                                    this.GPIOH1InputLowHighReadStateCheckBox.Checked = HighPinsInputData.bPin2LowHighState;
                                    this.GPIOH2InputLowHighReadStateCheckBox.Checked = HighPinsInputData.bPin3LowHighState;
                                    this.GPIOH3InputLowHighReadStateCheckBox.Checked = HighPinsInputData.bPin4LowHighState;
                                    this.GPIOH4InputLowHighReadStateCheckBox.Checked = HighPinsInputData.bPin5LowHighState;
                                    this.GPIOH5InputLowHighReadStateCheckBox.Checked = HighPinsInputData.bPin6LowHighState;
                                    this.GPIOH6InputLowHighReadStateCheckBox.Checked = HighPinsInputData.bPin7LowHighState;
                                    this.GPIOH7InputLowHighReadStateCheckBox.Checked = HighPinsInputData.bPin8LowHighState;
                                }
                            }
                        }
                    }

                    if ((ftStatus == FTC_SUCCESS) && (this.GPIOL1To3GPIOH0To7OnlyCheckBox.Checked == false))
                    {
                        //WriteControlBuffer[0] = 0xAE; //1 0 1 0 A2 A1 A0 0 - device address 7
                        // Modified device address to 1, to use the M24C64 EEPROM on the I2C Test Board With the 24C02(256 byte) EEPROM,
                        // M24C64(8192 byte) EEPROM, DS1621(Digital Thermometer and Thermostat) and M41T81(Serial Real-Time Clock)
                        WriteControlBuffer[0] = 0xA2; //1 0 1 0 A2 A1 A0 0 - device address 1

                        WriteDataBuffer[0] = 0xFF;  // Erase address locations

                        WriteDataByteAddress = 0;

                        PageWriteData.NumPages = 0;
                        PageWriteData.NumBytesPerPage = 0;

                        do
                        {
                            // shift down by 8 bits
                            WriteControlBuffer[1] = (byte)((WriteDataByteAddress >> 8) & 0xFF);
                            WriteControlBuffer[2] = (byte)(WriteDataByteAddress & 0xFF);

                            ftStatus = I2C_Write(ftHandle, WriteControlBuffer, 3, true, 20, true, BYTE_WRITE_TYPE, WriteDataBuffer, 1, true, 20, ref PageWriteData);

                            WriteDataByteAddress = WriteDataByteAddress + 1;
                        }
                        while ((WriteDataByteAddress < MAX_I2C_M24C64_CHIP_SIZE_IN_BYTES) && (ftStatus == FTC_SUCCESS));
                    }

                    //Console.WriteLine("Main thread - erased EEPROM.");

                    if ((ftStatus == FTC_SUCCESS) && (this.GPIOL1To3GPIOH0To7OnlyCheckBox.Checked == false)) {
                        //WriteControlBuffer[0] = 0xAE; //1 0 1 0 A2 A1 A0 0 - device address 7
                        WriteControlBuffer[0] = 0xA2; //1 0 1 0 A2 A1 A0 0 - device address 1

                        WriteDataByteAddress = 0;

                        DataWriteType = BYTE_WRITE_TYPE; //PAGE_WRITE_TYPE;

                        DataByte = 0x77;

                        if (DataWriteType == BYTE_WRITE_TYPE) {
                            WriteDataBuffer[0] = DataByte;

                            PageWriteData.NumPages = 0;
                            PageWriteData.NumBytesPerPage = 0;

                            NumDataBytesToWrite = 1;
                        } else {
                            WriteDataBuffer[0] = DataByte;

                            PageWriteData.NumPages = NUM_M24C64_PAGES;
                            PageWriteData.NumBytesPerPage = NUM_M24C64_BYTES_PER_PAGE;

                            NumDataBytesToWrite = (NUM_M24C64_PAGES * NUM_M24C64_BYTES_PER_PAGE);

                            for (WriteDataBufferIndex = 0; WriteDataBufferIndex < NumDataBytesToWrite; WriteDataBufferIndex++) {
                                WriteDataBuffer[WriteDataBufferIndex] = DataByte;
                            }
                        }

                        do
                        {
                            // performs an arithmetic right shift down by 8 bits
                            WriteControlBuffer[1] = (byte)((WriteDataByteAddress >> 8) & 0xFF);
                            WriteControlBuffer[2] = (byte)(WriteDataByteAddress & 0xFF);

                            ftStatus = I2C_Write(ftHandle, WriteControlBuffer, 3, true, 20, true, DataWriteType, WriteDataBuffer, NumDataBytesToWrite, true, 20, ref PageWriteData);

                            if (DataWriteType == BYTE_WRITE_TYPE) {
                                WriteDataByteAddress = WriteDataByteAddress + 1;
                            } else {
                                WriteDataByteAddress = WriteDataByteAddress + (NUM_M24C64_PAGES * NUM_M24C64_BYTES_PER_PAGE);
                            }
                        }
                        while ((WriteDataByteAddress < MAX_I2C_M24C64_CHIP_SIZE_IN_BYTES) && (ftStatus == FTC_SUCCESS));

                        //Console.WriteLine("Main thread - values written to EEPROM.");

                        if (ftStatus == FTC_SUCCESS) {
                            do {
                                //WriteControlBuffer[0] = 0xAF; //1 0 1 0 A2 A1 A0 1 - device address 7
                                WriteControlBuffer[0] = 0xA3; //1 0 1 0 A2 A1 A0 1 - device address 1

                                for (ReadDataIndex = 0; ReadDataIndex < MAX_I2C_M24C64_CHIP_SIZE_IN_BYTES; ReadDataIndex++) {
                                    ReadData[ReadDataIndex] = 0;
                                }

                                ReadDataByteAddress = 0;

                                DataReadType = BYTE_READ_TYPE; //BLOCK_READ_TYPE

                                if (DataReadType == BYTE_READ_TYPE) {
                                    NumDataBytesToRead = 1;
                                } else {
                                    NumDataBytesToRead = MAX_I2C_M24C64_CHIP_SIZE_IN_BYTES;
                                }

                                do
                                {
                                    // performs an arithmetic right shift down by 8 bits
                                    WriteControlBuffer[1] = (byte)((ReadDataByteAddress >> 8) & 0xFF);
                                    WriteControlBuffer[2] = (byte)(ReadDataByteAddress & 0xFF);

                                    ftStatus = I2C_Read(ftHandle, WriteControlBuffer, 3, true, 20, DataReadType, ReadDataBuffer, NumDataBytesToRead);

                                    if (ftStatus == FTC_SUCCESS) {
                                        if (DataReadType == BYTE_READ_TYPE) {
                                            ReadData[ReadDataByteAddress] = ReadDataBuffer[0];
                                        } else {
                                            for (ReadDataIndex = 0; ReadDataIndex < MAX_I2C_M24C64_CHIP_SIZE_IN_BYTES; ReadDataIndex++) {
                                                ReadData[ReadDataIndex] = ReadDataBuffer[ReadDataIndex];
                                            }
                                        }

                                        if (DataReadType == BYTE_READ_TYPE) {
                                            ReadDataByteAddress = ReadDataByteAddress + 1;
                                        } else {
                                            ReadDataByteAddress = ReadDataByteAddress + MAX_I2C_M24C64_CHIP_SIZE_IN_BYTES;
                                        }
                                    }
                                }
                                while ((ReadDataByteAddress < MAX_I2C_M24C64_CHIP_SIZE_IN_BYTES) && (ftStatus == FTC_SUCCESS));

                                //Console.WriteLine("Main thread - values read from EEPROM.");

                                if (ftStatus == FTC_SUCCESS) {
                                    ReadDataIndex = 0;

                                    do {
                                        if (DataByte != 0) {
                                            DataByteWritten = DataByte;
                                        } else {
                                            DataByteWritten = (byte)(ReadDataIndex & 0xFF);
                                        }

                                        if (((ReadData[ReadDataIndex] != DataByteWritten) && (DataByte != 0)) || ((ReadData[ReadDataIndex] != DataByteWritten) && (DataByte == 0)))
                                        {
                                            for (CharCntr = 0; CharCntr < 100; CharCntr++) {
                                                WriteDataBuffer[CharCntr] = 0;
                                            }

                                            mismatchMsg = String.Format("M24C64 Read/Write mismatch, Address = {0}, Write Value = {1}, Read Value = {2}", ReadDataIndex, DataByteWritten, ReadData[ReadDataIndex]);

                                            response = MessageBox.Show(mismatchMsg, "I2C Read Error Message", MsgBoxButtons);
                                        }

                                        ReadDataIndex = ReadDataIndex + 1;
                                    }
                                    while ((ReadDataIndex < MAX_I2C_M24C64_CHIP_SIZE_IN_BYTES) && (response == DialogResult.OK));
                                }

                                iLoopCntr = iLoopCntr + 1;
                            }
                            while ((iLoopCntr < 10) && (ftStatus == FTC_SUCCESS) && (response == DialogResult.OK));
                        }
                    }
                }
            }

            if (ftHandle != IntPtr.Zero)
            {
                //CloseFinalStatePinsData.bTCKPinState = true;
                //CloseFinalStatePinsData.bTCKPinActiveState = true;
                //CloseFinalStatePinsData.bTDIPinState = false;
                //CloseFinalStatePinsData.bTDIPinActiveState = false;
                //CloseFinalStatePinsData.bTMSPinState = false;
                //CloseFinalStatePinsData.bTMSPinActiveState = false;

                //ftStatus = I2C_CloseDevice(ftHandle, ref CloseFinalStatePinsData);

                I2C_Close(ftHandle);
                ftHandle = IntPtr.Zero;

                //Console.WriteLine("Main thread - device closed.");
            }

            if ((ftStatus != FTC_SUCCESS) || (numHiSpeedDevices == 0) || (mismatchMsg != null))
            {
                if (ftStatus != FTC_SUCCESS) {
                    ErrorMessage = "Status Code(" + ftStatus.ToString() + ") - ";

                    ftStatus = I2C_GetErrorCodeString("EN", ftStatus, byteErrorMessage, MAX_NUM_ERROR_MESSAGE_CHARS);

                    ErrorMessage = ErrorMessage + Encoding.ASCII.GetString(byteErrorMessage);
                    // Trim strings to first occurrence of a null terminator character
                    ErrorMessage = ErrorMessage.Substring(0, ErrorMessage.IndexOf("\0"));

                    MessageBox.Show(ErrorMessage);
                } else {
                    if (numHiSpeedDevices == 0)
                    {
                        ErrorMessage = "There are no devices connected.";

                        //MessageBox.Show(ErrorMessage);
                    }
                    else
                    {
                        ErrorMessage = mismatchMsg;
                    }
                }

                this.PassFailResultsStatusLabel.Text = "Fail - " + ErrorMessage;
                this.PassFailureStatusStrip.Refresh();

                //Console.WriteLine(ErrorMessage);
                Console.WriteLine("Fail - {0}", ErrorMessage);
            } else {
                this.PassFailResultsStatusLabel.Text = "Pass";
                this.PassFailureStatusStrip.Refresh();
            }
        }

        private void GPIOH0InputOutputInitialStateCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (this.GPIOH0InputOutputInitialStateCheckBox.Checked == false)
            {
                this.GPIOH0OutputLowHighInitialStateCheckBox.Enabled = false;
            } else {
                this.GPIOH0OutputLowHighInitialStateCheckBox.Enabled = true;
            }

            this.GPIOH0OutputLowHighInitialStateCheckBox.Checked = false;
        }

        private void GPIOH1InputOutputInitialStateCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (this.GPIOH1InputOutputInitialStateCheckBox.Checked == false)
            {
                this.GPIOH1OutputLowHighInitialStateCheckBox.Enabled = false;
            }
            else
            {
                this.GPIOH1OutputLowHighInitialStateCheckBox.Enabled = true;
            }

            this.GPIOH1OutputLowHighInitialStateCheckBox.Checked = false;
        }

        private void GPIOH2InputOutputInitialStateCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (this.GPIOH2InputOutputInitialStateCheckBox.Checked == false)
            {
                this.GPIOH2OutputLowHighInitialStateCheckBox.Enabled = false;
            }
            else
            {
                this.GPIOH2OutputLowHighInitialStateCheckBox.Enabled = true;
            }

            this.GPIOH2OutputLowHighInitialStateCheckBox.Checked = false;
        }

        private void GPIOH3InputOutputInitialStateCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (this.GPIOH3InputOutputInitialStateCheckBox.Checked == false)
            {
                this.GPIOH3OutputLowHighInitialStateCheckBox.Enabled = false;
            }
            else
            {
                this.GPIOH3OutputLowHighInitialStateCheckBox.Enabled = true;
            }

            this.GPIOH3OutputLowHighInitialStateCheckBox.Checked = false;
        }

        private void GPIOH4InputOutputInitialStateCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (this.GPIOH4InputOutputInitialStateCheckBox.Checked == false)
            {
                this.GPIOH4OutputLowHighInitialStateCheckBox.Enabled = false;
            }
            else
            {
                this.GPIOH4OutputLowHighInitialStateCheckBox.Enabled = true;
            }

            this.GPIOH4OutputLowHighInitialStateCheckBox.Checked = false;
        }

        private void GPIOH5InputOutputInitialStateCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (this.GPIOH5InputOutputInitialStateCheckBox.Checked == false)
            {
                this.GPIOH5OutputLowHighInitialStateCheckBox.Enabled = false;
            }
            else
            {
                this.GPIOH5OutputLowHighInitialStateCheckBox.Enabled = true;
            }

            this.GPIOH5OutputLowHighInitialStateCheckBox.Checked = false;
        }

        private void GPIOH6InputOutputInitialStateCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (this.GPIOH6InputOutputInitialStateCheckBox.Checked == false)
            {
                this.GPIOH6OutputLowHighInitialStateCheckBox.Enabled = false;
            }
            else
            {
                this.GPIOH6OutputLowHighInitialStateCheckBox.Enabled = true;
            }

            this.GPIOH6OutputLowHighInitialStateCheckBox.Checked = false;
        }

        private void GPIOH7InputOutputInitialStateCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (this.GPIOH7InputOutputInitialStateCheckBox.Checked == false)
            {
                this.GPIOH7OutputLowHighInitialStateCheckBox.Enabled = false;
            }
            else
            {
                this.GPIOH7OutputLowHighInitialStateCheckBox.Enabled = true;
            }

            this.GPIOH7OutputLowHighInitialStateCheckBox.Checked = false;
        }

        private void GPIOL1InputOutputInitialStateCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (this.GPIOL1InputOutputInitialStateCheckBox.Checked == false)
            {
                this.GPIOL1OutputLowHighInitialStateCheckBox.Enabled = false;
            }
            else
            {
                this.GPIOL1OutputLowHighInitialStateCheckBox.Enabled = true;
            }

            this.GPIOL1OutputLowHighInitialStateCheckBox.Checked = false;
        }

        private void GPIOL2InputOutputInitialStateCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (this.GPIOL2InputOutputInitialStateCheckBox.Checked == false)
            {
                this.GPIOL2OutputLowHighInitialStateCheckBox.Enabled = false;
            }
            else
            {
                this.GPIOL2OutputLowHighInitialStateCheckBox.Enabled = true;
            }

            this.GPIOL2OutputLowHighInitialStateCheckBox.Checked = false;
        }

        private void GPIOL3InputOutputInitialStateCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (this.GPIOL3InputOutputInitialStateCheckBox.Checked == false)
            {
                this.GPIOL3OutputLowHighInitialStateCheckBox.Enabled = false;
            }
            else
            {
                this.GPIOL3OutputLowHighInitialStateCheckBox.Enabled = true;
            }

            this.GPIOL3OutputLowHighInitialStateCheckBox.Checked = false;
        }

        private void MultipleThreadingCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (this.MultipleThreadingCheckBox.Checked == true)
            {
                this.ChannelComboBox.Text = "A";
            }
        }
    }
}